/*
 * tdsio.h
 *	writen by KIYOHARA Takashi <kiyohara@soum.co.jp>
 *	$SOUM$
 *  2007-08-17  modified by H.Ohkubo
 */

#ifndef _SYS_TDSIO_H_
#define _SYS_TDSIO_H_

#ifndef __KERNEL__
#include <stdint.h>
#endif

/* 
static const char _szVer[] = "Nitsuki tds_driver Ver1.2.1,2005-06-28";
*/
static const char _szVer[] = "Nitsuki tds_driver Ver1.2.2,2007-08-17";

/* TDS version 1 description,  9260 dependency (not recommended) */

/* sampling control */
#define TDS_SAMPLING_START	0x01
#define TDS_SAMPLING_STOP	0x00


/* interrupt */
#define TDS_INTR_NONE		0
#define TDS_INTR_1PPS		(0x01 << 0)
#define	TDS_INTR_DMA_DONE	(0x01 << 1)
#define TDS_INTR_DATA_EXIST	(0x01 << 3)
#define TDS_INTR_DATA_EMPTY	(0x01 << 4)
#define TDS_INTR_DATA_FULL	(0x01 << 5)
#define TDS_INTR_DATA_HALF	(0x01 << 6)			/* XXX: works fine ? */


/* sampling freqency */
#define TDS_MASK_SAMP_FREQ	0x0f
#define TDS_GET_SAMP_FREQ(arg)	((arg) & TDS_MASK_SAMP_FREQ)
#define TDS_SAMPLING_40KHZ	0x00
#define TDS_SAMPLING_100KHZ	0x01
#define TDS_SAMPLING_200KHZ	0x02
#define TDS_SAMPLING_500KHZ	0x03
#define TDS_SAMPLING_1MHZ	0x04
#define TDS_SAMPLING_2MHZ	0x05
#define TDS_SAMPLING_4MHZ	0x06
#define TDS_SAMPLING_8MHZ	0x07
#define TDS_SAMPLING_16MHZ	0x08
#define TDS_SAMPLING_32MHZ	0x09
#define TDS_SAMPLING_64MHZ	0x0a


/* sampling width */
#define TDS_MASK_SAMP_WIDTH	(0x03 << 4)
#define TDS_GET_SAMP_WIDTH(arg)	((arg) & TDS_MASK_SAMP_WIDTH)
#define TDS_SAMPLING_1BIT	(0x00 << 4)
#define TDS_SAMPLING_2BIT	(0x01 << 4)
#define TDS_SAMPLING_4BIT	(0x02 << 4)
#define TDS_SAMPLING_8BIT	(0x03 << 4)


/* sampling channel */
#define TDS_MASK_SAMP_CH	(0x01 << 6)
#define TDS_GET_SAMP_CH(arg)	((arg) & TDS_MASK_SAMP_CH)
#define TDS_SAMPLING_1CH	(0x00 << 6)
#define TDS_SAMPLING_4CH	(0x01 << 6)


/* time sync request / input signal check */
#define TDS_1PPS_SYNC		(0x01 << 0)
#define TDS_TIME_SYNC		(0x01 << 1)
#define TDS_SIGNAL_CHECK	(0x01 << 2)


/* board status */
#define TDS_STAT_10MHZ_INPUT	(0x01 << 0)
#define TDS_STAT_1PPS_INPUT	(0x01 << 1)
#define TDS_STAT_TIME_INPUT	(0x01 << 2)
#define TDS_STAT_SAMPLING	(0x01 << 3)
#define TDS_STAT_FIFO_EMPTY	(0x01 << 4)
#define TDS_STAT_FIFO_FULL	(0x01 << 5)
#define TDS_STAT_FIFO_HALF	(0x01 << 6)
#define TDS_STAT_FIFO_OVERFLOW	(0x01 << 7)


/* data input status */
#define	TDS_INPUT_MAIN		(0x01 << 0)
#define TDS_INPUT_RES1		(0x01 << 1)		/* reserve */
#define TDS_INPUT_RES2		(0x01 << 2)		/* reserve */
#define TDS_EXTCARD_EXIST	(0x01 << 3)
#define TDS_INPUT_EXT1		(0x01 << 4)
#define TDS_INPUT_EXT2		(0x01 << 5)
#define TDS_INPUT_EXT3		(0x01 << 6)
#define TDS_INPUT_EXT4		(0x01 << 7)


/* TDS version 2 description,  genericaly (recommended) */

/* sampling freqency */
#define TDSIO_SAMPLING_40KHZ	0x00
#define TDSIO_SAMPLING_100KHZ	0x01
#define TDSIO_SAMPLING_200KHZ	0x02
#define TDSIO_SAMPLING_500KHZ	0x03
#define TDSIO_SAMPLING_1MHZ	0x04
#define TDSIO_SAMPLING_2MHZ	0x05
#define TDSIO_SAMPLING_4MHZ	0x06
#define TDSIO_SAMPLING_8MHZ	0x07
#define TDSIO_SAMPLING_16MHZ	0x08
#define TDSIO_SAMPLING_32MHZ	0x09
#define TDSIO_SAMPLING_64MHZ	0x0a

/* sampling width */
#define TDSIO_SAMPLING_1BIT	0x00
#define TDSIO_SAMPLING_2BIT	0x01
#define TDSIO_SAMPLING_4BIT	0x02
#define TDSIO_SAMPLING_8BIT	0x03

/* sampling channel */
#define TDSIO_SAMPLING_1CH	0x00
#define TDSIO_SAMPLING_4CH	0x01

/* sampling filter */
#define TDSIO_SAMPLING_THRU	0x00
#define TDSIO_SAMPLING_16M	0x01
#define TDSIO_SAMPLING_8M	0x02
#define TDSIO_SAMPLING_4M	0x03
#define TDSIO_SAMPLING_2M	0x04

/* time sync request / input signal check */
#define TDSIO_1PPS_SYNC		(0x01 << 0)
#define TDSIO_TIME_SYNC		(0x01 << 1)
#define TDSIO_SIGNAL_CHECK	(0x01 << 2)

/* board status */
#define TDSIO_STAT_10MHZ_INPUT		(0x01 << 0)
#define TDSIO_STAT_1PPS_INPUT		(0x01 << 1)
#define TDSIO_STAT_TIME_INPUT		(0x01 << 2)
#define TDSIO_STAT_SAMPLING		(0x01 << 3)
#define TDSIO_STAT_FIFO_EMPTY		(0x01 << 4)
#define TDSIO_STAT_FIFO_FULL		(0x01 << 5)
#define TDSIO_STAT_FIFO_HALF		(0x01 << 6)
#define TDSIO_STAT_FIFO_OVERFLOW	(0x01 << 7)

/* data input status */
#define	TDSIO_INPUT_MAIN	(0x01 << 0)
#define TDSIO_INPUT_RES1	(0x01 << 1)		/* reserve */
#define TDSIO_INPUT_RES2	(0x01 << 2)		/* reserve */
#define TDSIO_EXTCARD_EXIST	(0x01 << 3)
#define TDSIO_INPUT_EXT1	(0x01 << 4)
#define TDSIO_INPUT_EXT2	(0x01 << 5)
#define TDSIO_INPUT_EXT3	(0x01 << 6)
#define TDSIO_INPUT_EXT4	(0x01 << 7)


/* TDS driver mode */
#define TDS_MODE_TIMER_DRIVEN	1
#define TDS_MODE_INTR_DRIVEN	2
#define TDS_MODE_DEFAULT	TDS_MODE_INTR_DRIVEN


/* program I/O or DMA */
#define	TDS_READ_USE_PIO	0	/* programmed I/O */
#define	TDS_READ_USE_DMA	1	/* bus master transfer */
#define	TDS_READ_DEFAULT	TDS_READ_USE_DMA


struct tdsio {
	uint32_t key;
	uint32_t value;
};
typedef struct tdsio *tdsio_t;

struct tdsauxio {
	int auxsize;
	char auxfield[256];
};
typedef struct tdsauxio *tdsauxio_t;

struct tdsio_regio {
	unsigned long address;
	unsigned long value;
};
typedef struct tdsio_regio *tdsio_regio_t;


/* ioctl() command code */
#define TDS_IO_GROUP			'N'
#define TDSIO_INIT			_IO  (TDS_IO_GROUP, 0x01)
#define TDSIO_BUFFER_CLEAR		_IO  (TDS_IO_GROUP, 0x02)
#define TDSIO_SYNC_1PPS			_IO  (TDS_IO_GROUP, 0x03)
#define TDSIO_SYNC_TIME			_IO  (TDS_IO_GROUP, 0x04)
#define TDSIO_SET_INTERRUPT		_IOW (TDS_IO_GROUP, 0x05, uint32_t *)	/* obsolete */
#define TDSIO_GET_INTERRUPT		_IOR (TDS_IO_GROUP, 0x06, uint32_t *)	/* obsolete */
#define TDSIO_SET_TIME			_IOW (TDS_IO_GROUP, 0x07, uint32_t *)
#define TDSIO_GET_TIME			_IOR (TDS_IO_GROUP, 0x08, uint32_t *)
#define TDSIO_GET_YEAR			_IOR (TDS_IO_GROUP, 0x09, uint32_t *)
#define TDSIO_SET_SAMPLING		_IOW (TDS_IO_GROUP, 0x0a, uint32_t *)
#define TDSIO_GET_SAMPLING		_IOR (TDS_IO_GROUP, 0x0b, uint32_t *)
#define TDSIO_GET_STATUS		_IOR (TDS_IO_GROUP, 0x0c, uint32_t *)
#define TDSIO_SAMPLING_START		_IO  (TDS_IO_GROUP, 0x0d)
#define TDSIO_SAMPLING_STOP		_IO  (TDS_IO_GROUP, 0x0e)
#define TDSIO_GET_ERROR			_IOR (TDS_IO_GROUP, 0x0f, uint32_t *)
#define TDSIO_SET_VERBOSE		_IOW (TDS_IO_GROUP, 0x11, int *)
#define TDSIO_SET_MODE			_IOW (TDS_IO_GROUP, 0x12, int *)	/* obsolete */
#define TDSIO_GET_MODE			_IOR (TDS_IO_GROUP, 0x13, int *)		/* obsolete */
#define TDSIO_SET_ALLOW_OVERFLOW	_IOW (TDS_IO_GROUP, 0x14, int *)
#define TDSIO_GET_OVERFLOW_COUNT	_IOR (TDS_IO_GROUP, 0x15, int *)
#define	TDSIO_SET_READ_MODE		_IOW (TDS_IO_GROUP, 0x16, int *)
#define TDSIO_GET_READ_MODE		_IOR (TDS_IO_GROUP, 0x17, int *)
#define	TDSIO_GET_INPUT_STATUS		_IOR (TDS_IO_GROUP, 0x18, int *)
#define	TDSIO_GET_DMA_STATUS		_IOR (TDS_IO_GROUP, 0x19, int *)
#define TDSIO_SET_DCOFFSET		_IOWR(TDS_IO_GROUP, 0x1a, tdsio_t)
#define TDSIO_GET_DCOFFSET		_IOWR(TDS_IO_GROUP, 0x1b, tdsio_t)
#define TDSIO_BUFFER_RESET		_IO  (TDS_IO_GROUP, 0x1c)
#define	TDSIO_GET_1PPSINPUT		_IOR (TDS_IO_GROUP, 0x1d, int *)
#define	TDSIO_GET_REFINPUT		_IOR (TDS_IO_GROUP, 0x1e, int *)
#define	TDSIO_GET_10MHZINPUT		_IOR (TDS_IO_GROUP, 0x1f, int *)
#define	TDSIO_SET_CHMODE		_IOW (TDS_IO_GROUP, 0x20, uint32_t *)
#define	TDSIO_GET_CHMODE		_IOR (TDS_IO_GROUP, 0x21, uint32_t *)
#define	TDSIO_SET_RESOLUTIONBIT		_IOW (TDS_IO_GROUP, 0x22, uint32_t *)
#define	TDSIO_GET_RESOLUTIONBIT		_IOR (TDS_IO_GROUP, 0x23, uint32_t *)
#define	TDSIO_SET_FILTER		_IOW (TDS_IO_GROUP, 0x24, uint32_t *)
#define	TDSIO_GET_FILTER		_IOR (TDS_IO_GROUP, 0x25, uint32_t *)
#define	TDSIO_SET_FSAMPLE		_IOW (TDS_IO_GROUP, 0x26, uint32_t *)
#define	TDSIO_GET_FSAMPLE		_IOR (TDS_IO_GROUP, 0x27, uint32_t *)
#define TDSIO_SET_YEAR			_IOR (TDS_IO_GROUP, 0x28, uint32_t *)
#define TDSIO_SET_AUX			_IOW (TDS_IO_GROUP, 0x29, tdsauxio_t)
#define TDSIO_GET_AUX			_IOR (TDS_IO_GROUP, 0x2a, tdsauxio_t)

#ifdef UTDS_DO_GARBAGE_CLEAR
#define TDSIO_GET_REFOUND		_IOR (TDS_IO_GROUP, 0xc0, uint32_t *)
#endif


/*2007/8/18 H.Ohkubo*/
#define TDSIO_SET_BITSHIFT		_IOWR(TDS_IO_GROUP, 0xf8, tdsio_t)
#define TDSIO_GET_BITSHIFT		_IOWR(TDS_IO_GROUP, 0xf9, tdsio_t)
#define TDSIO_GET_VERFPGA		_IOWR(TDS_IO_GROUP, 0xfa, tdsio_t)


#define TDSIO_GET_VERSION		_IOR (TDS_IO_GROUP, 0xfb, uint32_t *)
#define TDSIO_REGIO_READ		_IOWR(TDS_IO_GROUP, 0xfc, tdsio_regio_t)
#ifdef  TDS_DEBUG
#define TDSIO_REGIO_WRITE		_IOW (TDS_IO_GROUP, 0xfd, tdsio_regio_t)
#define TDSIO_GET_DEBUG			_IOR (TDS_IO_GROUP, 0xfe, uint32_t *)
#define TDSIO_SET_DEBUG			_IOW (TDS_IO_GROUP, 0xff, uint32_t *)
#endif


#define TDS_VERSION_MAJOR(x)	(((x) >> 16) & 0xff)
#define TDS_VERSION_MINOR(x)	(((x) >> 8) & 0xff)
#define TDS_VERSION_REVISION(x)	((x) & 0xff)


/* time / date */
#define TDS_SEC_MASK		(0x0001ffff)
#define TDS_SEC_CARRY_MASK	(0x01 << 17)
#define TDS_DAYS_MASK		(0x01ff << 18)
#define TDS_DAYS_CARRY_MASK	(0x01 << 27)
#define TDS_GET_SEC(arg)	((arg) & TDS_SEC_MASK)
#define TDS_GET_SEC_CARRY(arg)	(((arg) & TDS_SEC_CARRY_MASK) >> 17)
#define TDS_GET_DAYS(arg)	(((arg) & TDS_DAYS_MASK) >> 18)
#define TDS_GET_DAYS_CARRY(arg)	(((arg) & TDS_DAYS_CARRY_MASK) >> 27)
#define TDS_MAKE_TIME(day,sec)	\
	((((day) << 18) & TDS_DAYS_MASK) | ((sec) & TDS_SEC_MASK))


/* error code for tds */
#define TDS_ERROR_NO_ERROR		0
#define TDS_ERROR_UNKNOWN		1
#define TDS_ERROR_NOT_SUPPORTED		2
#define TDS_ERROR_NOT_STARTED		3
#define TDS_ERROR_ALREADY_STARTED	4
#define TDS_ERROR_NOT_SET_SAMPLING	5
#define TDS_ERROR_FIFO_OVERFLOW		6
#define TDS_ERROR_1PPS_NOT_DETECT	7
#define TDS_ERROR_10MHZ_NOT_DETECT	8
#define TDS_ERROR_TIME_NOT_DETECT	9
#define TDS_ERROR_NOT_SYNC_1PPS		10
#define TDS_ERROR_NOT_SYNC_TIME		11
#define TDS_ERROR_READ_BUSY		12
#define TDS_ERROR_EXTCARD_NOT_EXIST	13
#define TDS_ERROR_NO_DMA_BUFFER		14
#define TDS_ERROR_IO_ERROR		15
#define TDS_ERROR_DMA_ERROR		16
#define TDS_ERROR_TIMEOUT		17
#define TDS_ERROR_NOT_ALIGNED		18
#define TDS_ERROR_NO_HEADER		19
#define TDS_ERROR_RE_HEADER		20


#ifdef _KERNEL


#else	/* _KERNEL */


struct tdsdev {
	char *dev;
	int fd;
	int error;
	char *type;
};

typedef struct tdsdev tdsdev_t;


struct tds_time {
	int	sec;
	int sec_carry;
	int day;
	int day_carry;
};

typedef struct tds_time tds_time_t;


struct tds_samp {
	int freq;
	int unit;	/* 'k' or 'M' */
	int width;
	int channel;
};

typedef struct tds_samp tds_samp_t;


tdsdev_t *TdsOpen(char *);
tdsdev_t *TdsOpen2(char *, char *);
int TdsClose(tdsdev_t *);
int TdsInit(tdsdev_t *);
int TdsClrBuffer(tdsdev_t *);
int TdsSync1pps(tdsdev_t *);
int TdsSyncTime(tdsdev_t *);
int TdsSetTime(tdsdev_t *, tds_time_t *);
int TdsGetTime(tdsdev_t *, tds_time_t *);
int TdsSetYear(tdsdev_t *, unsigned int *);
int TdsGetYear(tdsdev_t *, unsigned int *);
int TdsSetSampling(tdsdev_t *, tds_samp_t *);
int TdsGetSampling(tdsdev_t *, tds_samp_t *);
int TdsSetSampling2(tdsdev_t *, int *, int *, int *, int *);
int TdsGetSampling2(tdsdev_t *, int *, int *, int *, int *);
int TdsSetDcoffset(tdsdev_t *, int *, unsigned int *);
int TdsGetDcoffset(tdsdev_t *, int *, unsigned int *);
int TdsStatus(tdsdev_t *, unsigned int *);
int TdsStart(tdsdev_t *);
int TdsStop(tdsdev_t *);
int TdsGetData(tdsdev_t *, unsigned int *);
int TdsRead(tdsdev_t *, int *, unsigned char *);
int TdsSetAux(tdsdev_t *, int *, char *);
int TdsSetAuxFile(tdsdev_t *, char *);
int TdsGetAux(tdsdev_t *, int *, char *);
int TdsGetAuxFile(tdsdev_t *, char *);
#ifdef UTDS_DO_GARBAGE_CLEAR
int TdsGetRefound(tdsdev_t *, unsigned int *);
#endif
int TdsError(tdsdev_t *, int *, char **);
int TdsGetVersion(tdsdev_t *, int *, int *);

#endif	/*  _KERNEL */


#endif	/* ! _SYS_TDSIO_H_ */
